{% spaceless %}

{% compress js %}
    <script type="text/javascript">
        alert("Basic test, should pass in spite of errors in other templates");
    </script>
{% endcompress %}
{% endspaceless %}
